/*
body {
	font: 85% Verdana,Arial,sans-serif;
	padding: 1em;
}
h1, h2, h3 {
	color: #339;
}
h1 {
	font-size: 130%;
	margin: 0 0 1em;
}
h2 {
	font-size: 110%;
	margin: 0.75em 0 0.5em;
}
h3 {
	font-size: 100%;
	margin: 0.5em 0 0.25em;
}
p {
	margin: 0.35em 0;
}
ol, ul, li {
	margin-top: 0.25em;
	margin-bottom: 0.25em;
}
pre {
	white-space: pre;
	color: #060;
	font: 90% Courier New,monospace;
	margin: 0.75em;
}
kbd {
	font: 90% Courier New,monospace;
	color: #060;
}
code {
	font: 100% Courier New,monospace;
	color: #050;
}
em {
	color: #833;
	font-weight: bold;
}
dt {
	color: #336;
	font-weight: bold;
	margin: 0.25em 0;
}
dd {
	margin: 0.25em 2em 0.5em;
}
*/

html, body
{
	margin: 0;			/* Remove body margin/padding */
	padding: 0;
	overflow: hidden;	/* Remove scroll bars on browser window */
}
#header { padding: 1em; }

.moksha-feedreader {
	border: 3px solid #669;
	min-width: 250px;	/* Splitter can't be too thin ... */
	min-height: 525px;	/* ... or too flat */
    max-height: 525px;
	height: 600px;
    width: 850px;
}
#LeftPane {
	background: #ddf;
	width: 150px;
	min-width: 250px;
	max-width: 250px;
	overflow: auto;		/* Scroll bars appear as needed */
}
#TopPane {				/* Top nested in right pane */
	background: #ccf;
	height: 200px;		/* Initial height */
	min-height: 75px;	/* Minimum height */
	overflow: auto;	
}
#BottomPane {			/* Bottom nested in right pane */
	background: #eef;
	min-height: 100px;
	overflow: auto;
}

/* Splitbar styles; these are the default class names */

.vsplitbar {
	width: 6px;
	background: #669 url(vgrabber.gif) no-repeat center;
}
.vsplitbar:hover, .vsplitbar.active {
	background: #c66 url(vgrabber.gif) no-repeat center;
	opacity: 0.7;
	filter: alpha(opacity=70); /* IE */
	background: #c99;
}
.hsplitbar {
	height: 6px;
	background: #669 url(hgrabber.gif) no-repeat center;
}
.hsplitbar.active, .hsplitbar:hover {
	background: #c66 url(hgrabber.gif) no-repeat center;
}
